@@ -1,17 +1,18 @@
-#' Data.frame-like operations on sparse and dense \code{Matrix} objects
+#' Data.frame-Like Operations on Sparse and Dense \code{Matrix} Objects
 #' 
-#' Implements cast, aggregate, and join for \code{\link{Matrix}} and matrix-like
-#' objects.
+#' Implements data manipulation methods such as cast, aggregate, and merge/join
+#' for \code{\link{Matrix}} and matrix-like objects.
 #' 
 #' @name Matrix.utils
 #' @docType package
 #' @import Matrix
+#' @import grr
 #' @importFrom stats aggregate
-#' @importFrom methods is
+#' @importFrom methods is as
 #' @importFrom stats as.formula terms
 NULL
 
-#' Casts or pivots a long \code{data frame} into a wide sparse matrix.
+#' Casts or pivots a long \code{data frame} into a wide sparse matrix
 #' 
 #' Similar in function to \code{\link[reshape2]{dcast}}, but produces a sparse 
 #' \code{\link{Matrix}} as an output. Sparse matrices are beneficial for this 
@@ -33,6 +34,7 @@ NULL
 #' @param fun.aggregate name of aggregation function.  Defaults to 'sum'
 #' @param value.var name of column that stores values to be aggregated numerics
 #' @param as.factors if TRUE, treat all columns as factors, including
+#' @return a sparse \code{Matrix}
 #' @seealso \code{\link[reshape]{cast}}
 #' @seealso \code{\link[reshape2]{dcast}}
 #' @export
@@ -128,7 +130,7 @@ dMcast<-function(data,formula,fun.aggregate='sum',value.var=NULL,as.factors=FALS
   return(result)
 }
 
-#' Compute summary statistics of a Matrix.
+#' Compute summary statistics of a Matrix
 #' 
 #' Similar to \code{\link[stats]{aggregate}}.  Splits the matrix into groups as
 #' specified by groupings, which can be one or more variables. Aggregation
@@ -183,58 +185,201 @@ aggregate.Matrix<-function(x,groupings=NULL,form=NULL,fun='sum',...)
   return(result)
 }
 
-#'Merges two Matrices or matrix-like objects 
+
+#'Merges two Matrices or matrix-like objects
 #'
-#' Similar to \code{\link{merge}} or \code{\link[plyr]{join}}. Currently can only do left and 
-#' inner joins that only find the first matching row; equivalent to 
-#' plyr::join(match='first').  This is sufficient for N to 1 joins, such as joining a list of
-#' addresses to a zip code lookup table.
+#'Implementation of \code{\link{merge}} for \code{\link{Matrix}}.  By explicitly
+#'calling \code{merge.Matrix} it will also work for \code{matrix}, for 
+#'\code{data.frame}, and \code{vector} objects as a much faster alternative to 
+#'the built-in \code{merge}.
 #'
-#'@param x \code{\link{Matrix}} or matrix-like object
+#'#' \code{all.x/all.y} correspond to the four types of database joins in the 
+#'following way:
+#'
+#'\describe{ \item{left}{\code{all.x=TRUE}, \code{all.y=FALSE}} 
+#'\item{right}{\code{all.x=FALSE}, \code{all.y=TRUE}} 
+#'\item{inner}{\code{all.x=FALSE}, \code{all.y=FALSE}} 
+#'\item{full}{\code{all.x=TRUE}, \code{all.y=TRUE}} }
+#'
+#'Note that \code{NA} values will match other \code{NA} values.
+#'
+#'@param x \code{Matrix} or matrix-like object
 #'@param y \code{Matrix} or matrix-like object
 #'@param by.x vector indicating the names to match from \code{Matrix} x
 #'@param by.y vector indicating the names to match from \code{Matrix} y
-#'@param type type of join: currently on left and inner are supported
+#'@param all.x logical; if \code{TRUE}, then each value in \code{x} will be 
+#'  included even if it has no matching values in \code{y}
+#'@param all.y logical; if \code{TRUE}, then each value in \code{y} will be 
+#'  included even if it has no matching values in \code{x}
 #'@param ... arguments to be passed to or from methods.  Currently ignored
 #'@export
 #'@export merge.Matrix
-#' @examples
-#' orders<-Matrix(as.matrix(data.frame(orderNum=1:1000,
-#'    customer=sample(100,1000,TRUE))),sparse=TRUE)
-#' cancelledOrders<-Matrix(as.matrix(data.frame(orderNum=sample(1000,100),
-#'    cancelled=1)),sparse=TRUE)
-#' skus<-Matrix(as.matrix(data.frame(orderNum=sample(1000,10000,TRUE),
-#'    sku=sample(1000,10000,TRUE),
-#'    amount=runif(10000))),sparse=TRUE)
-#' a<-merge.Matrix(orders,cancelledOrders,orders[,'orderNum'],cancelledOrders[,'orderNum'])
-#' b<-merge.Matrix(orders,cancelledOrders,orders[,'orderNum'],cancelledOrders[,'orderNum'],
-#'    type='inner')
-#' c<-merge.Matrix(orders,skus,orders[,'orderNum'],skus[,'orderNum'])
+#'@examples
 #' 
-#' \dontrun{
-#' ## Not run:
-#' orders<-data.frame(orderNum=sample(1e6, 1e7, TRUE),
-#'    sku=sample(1e3, 1e7, TRUE),
-#'    customer=sample(1e4,1e7,TRUE))
-#' cancelledOrders<-data.frame(data.frame(orderNum=sample(1e6,1e5),cancelled=1))
-#' system.time(b<-merge.Matrix(orders,cancelledOrders,orders[,'orderNum'],
-#'    cancelledOrders[,'orderNum'],type='inner'))
-#' #The following is the equivalent call in plyr, but returns an error due to a bug in plyr
-#' system.time(c<-plyr::join(orders,cancelledOrders,type='inner',match='first')) 
+#' orders<-Matrix(as.matrix(data.frame(orderNum=1:1000, 
+#' customer=sample(100,1000,TRUE)))) 
+#' cancelledOrders<-Matrix(as.matrix(data.frame(orderNum=sample(1000,100), 
+#' cancelled=1))) 
+#' skus<-Matrix(as.matrix(data.frame(orderNum=sample(1000,10000,TRUE), 
+#' sku=sample(1000,10000,TRUE), amount=runif(10000)))) 
+#' a<-merge(orders,cancelledOrders,orders[,'orderNum'],cancelledOrders[,'orderNum'])
+#' b<-merge(orders,cancelledOrders,orders[,'orderNum'],cancelledOrders[,'orderNum'],all.x=FALSE)
+#' c<-merge(orders,skus,orders[,'orderNum'],skus[,'orderNum'])
+#' 
+#' #The above Matrices could be converted to matrices or data.frames and handled in other methods.  
+#' #However, this is not possible in the sparse case, which can be handled by this function:
+#' sm<-cbind2(1:200000,rsparsematrix(200000,10000,density=.0001))
+#' sm2<-cbind2(sample(1:200000,50000,TRUE),rsparsematrix(200000,10,density=.01))
+#' sm3<-merge.Matrix(sm,sm2,by.x=sm[,1],by.y=sm2[,1])
+#' 
+#'  \dontrun{
+#' #merge.Matrix can also handle many other data types, such as data frames, and is generally fast.
+#' orders<-data.frame(orderNum=as.character(sample(1e5, 1e6, TRUE)),
+#'    sku=sample(1e3, 1e6, TRUE),
+#'    customer=sample(1e4,1e6,TRUE),stringsAsFactors=FALSE)
+#' cancelledOrders<-data.frame(orderNum=as.character(sample(1e5,1e4)),
+#'    cancelled=1,stringsAsFactors=FALSE)
+#' system.time(a<-merge.Matrix(orders,cancelledOrders,orders[,'orderNum'],
+#'    cancelledOrders[,'orderNum']))
+#' system.time(b<-merge.data.frame(orders,cancelledOrders,all.x = TRUE,all.y=TRUE))
+#' system.time(c<-dplyr::full_join(orders,cancelledOrders))
+#'system.time({require(data.table);
+#'d<-merge(data.table(orders),data.table(cancelledOrders),
+#'    by='orderNum',all=TRUE,allow.cartesian=TRUE)})
+#'
+#'orders<-data.frame(orderNum=sample(1e5, 1e6, TRUE), sku=sample(1e3, 1e6,
+#'TRUE), customer=sample(1e4,1e6,TRUE),stringsAsFactors=FALSE) 
+#'cancelledOrders<-data.frame(orderNum=sample(1e5,1e4),cancelled=1,stringsAsFactors=FALSE)
+#'system.time(b<-merge.Matrix(orders,cancelledOrders,orders[,'orderNum'], 
+#'cancelledOrders[,'orderNum'])) 
+#'system.time(e<-dplyr::full_join(orders,cancelledOrders)) 
+#'system.time({require(data.table);
+#'  d<-merge(data.table(orders),data.table(cancelledOrders),
+#'  by='orderNum',all=TRUE,allow.cartesian=TRUE)})
+#'
+#'#In certain cases, merge.Matrix can be much faster than alternatives. 
+#'one<-as.character(1:1000000) two<-as.character(sample(1:1000000,1e5,TRUE)) 
+#'system.time(b<-merge.Matrix(one,two,one,two)) 
+#'system.time(c<-dplyr::full_join(data.frame(key=one),data.frame(key=two))) 
+#'system.time({require(data.table);
+#'  d<-merge(data.table(data.frame(key=one)),data.table(data.frame(key=two)),
+#'  by='key',all=TRUE,allow.cartesian=TRUE)})
 #'}
-merge.Matrix<-function(x,y,by.x=rownames(x),by.y=rownames(y),type='left',...)
+merge.Matrix<-function(x,y,by.x,by.y,all.x=TRUE,all.y=TRUE,...)
 {
-  indices1<-match(by.x,by.y)
-  indices1[is.na(indices1)]<-nrow(y)+1
-  y2<-rbind2(y,NA)[indices1,]
-  colnames(y2)[colnames(y2) %in% colnames(x)]<-paste('y',colnames(y2)[colnames(y2) %in% colnames(x)],sep='.')
-  result<-cbind2(x,y2)
-  if(type=='inner')
-    result<-result[!(indices1==(nrow(y)+1)),]
+  requireNamespace('grr')
+  if(is.null(dim(x)))
+    return(grr::matches(by.x,by.y,all.x,all.y,indexes=FALSE))
+  indices<-grr::matches(by.x,by.y,all.x,all.y,nomatch = NULL)
+  x<-rBind(x,NA)
+  y<-rBind(y,NA)
+  if(!is.null(colnames(x)) & !is.null(colnames(y)))
+    colnames(y)[colnames(y) %in% colnames(x)]<-paste('y',colnames(y)[colnames(y) %in% colnames(x)],sep='.')
+  result<-cbind2(grr::extract(x,indices$x),grr::extract(y,indices$y))
   return(result)
 }
 
-#' Synonym for merge.Matrix.
 #' @rdname merge.Matrix
 #' @export
-join.Matrix<-merge.Matrix
\ No newline at end of file
+join.Matrix<-merge.Matrix
+
+#' Combine matrixes by row, fill in missing columns
+#' 
+#' rbinds a list of Matrix or matrix like objects, filling in missing columns.
+#' 
+#' Similar to \code{\link[plyr]{rbind.fill.matrix}}, but works for
+#' \code{\link{Matrix}} as well as all other R objects.  It is completely
+#' agnostic to class, and will produce an object of the class of the first input
+#' (or of class \code{matrix} if the first object is one dimensional).
+#' 
+#' The implementation is recursive, so it can handle an arbitrary number of 
+#' inputs, albeit inefficiently for large numbers of inputs.
+#' 
+#' This method is still experimental, but should work in most cases.  If the
+#' data sets consist solely of data frames, \code{\link[plyr]{rbind.fill}} is 
+#' preferred.
+#' 
+#' @param x,... Objects to combine.  If the first argument is a list and 
+#'   \code{..} is unpopulated, the objects in that list will be combined.
+#' @param fill value with which to fill unmatched columns
+#' @return a single object of the same class as the first input, or of class
+#'   \code{matrix} if the first object is one dimensional
+#' @seealso \code{\link[plyr]{rbind.fill}}
+#' @seealso \code{\link[plyr]{rbind.fill.matrix}}
+#' @export
+#' @examples
+#' df1 = data.frame(x = c(1,2,3), y = c(4,5,6))
+#' rownames(df1) = c("a", "b", "c")
+#' df2 = data.frame(x = c(7,8), z = c(9,10))
+#' rownames(df2) = c("a", "d")
+#' rBind.fill(df1,df2,fill=NA)
+#' rBind.fill(as(df1,'Matrix'),df2,fill=0)
+#' rBind.fill(as.matrix(df1),as(df2,'Matrix'),c(1,2),fill=0)
+#' rBind.fill(c(1,2,3),list(4,5,6,7))
+#' rBind.fill(df1,c(1,2,3,4))
+#' 
+#' m<-rsparsematrix(1000000,100,.001)
+#' m2<-m
+#' colnames(m)<-1:100
+#' colnames(m2)<-3:102
+#' system.time(b<-rBind.fill(m,m2))
+#' 
+rBind.fill<-function(x,...,fill=NA)
+{
+  if (is.list(x) && !is.data.frame(x) && missing(...)) {
+    Reduce(function (x,y) rBind.fill.internal(x,y,fill),x)
+  }
+  else {
+    Reduce(function (x,y) rBind.fill.internal(x,y,fill),list(x,...))
+  }
+}
+
+rBind.fill.internal<-function(x,y,fill=NA)
+{
+  if (is.null(nrow(x)))
+       x<-matrix(x,nrow=1,dimnames=list(NULL,names(x)))
+  if (is.null(nrow(y)))
+      y<-matrix(y,nrow=1,dimnames=list(NULL,names(y)))
+  y<-
+  {
+    if('data.frame' %in% is(x) && ('Matrix' %in% is(y)))
+      as.data.frame(y)
+    else
+    if('Matrix' %in% is(x))
+      as(y,'Matrix')
+    else
+      y
+  }
+
+  nullNames<-FALSE
+  if(is.null(colnames(x)))
+    colnames(x)<-colnames(y)[1:ncol(x)]
+  if(is.null(colnames(y)))
+     colnames(y)<-colnames(x)[1:ncol(y)]
+  if(is.null(colnames(x)))
+  {
+    nullNames<-TRUE
+    colnames(x)<-1:ncol(x)
+    colnames(y)<-1:ncol(y)
+  }
+  ymiss<-colnames(x)[which(is.na(match(colnames(x),colnames(y))))]
+  ybind<-matrix(fill,nrow=nrow(y),ncol=length(ymiss))
+  colnames(ybind)<-ymiss
+  xmiss<-colnames(y)[which(is.na(match(colnames(y),colnames(x))))]
+  xbind<-matrix(fill,nrow=nrow(x),ncol=length(xmiss))
+  colnames(xbind)<-xmiss
+  x<-cbind2(x,xbind)
+  y<-cbind2(y,ybind)
+  result<-rbind2(x,y[,order(match(colnames(y),colnames(x)))])
+  if(nullNames)
+    colnames(result)<-NULL
+  return(result)
+}
+
+len<-function (data) 
+{
+  result <- ifelse(is.null(nrow(data)), length(data), nrow(data))
+  return(result)
+}
+
+setAs('Matrix','data.frame',function (from) as.data.frame(as.matrix(from)))
